import 'dart:io';
import 'package:yona/yona.dart';
import 'dart:math' as math;

var host = NetAddr.fromString("yonafediverse.com:7777");
var n = 10;
void clearConsole(){
  if(Platform.isWindows) {
    print(Process.runSync("cls", [], runInShell: true).stdout);
  } else {
    print(Process.runSync("clear", [], runInShell: true).stdout);
  }

}
void main() async {
  var user1 = await Crypto.genSK();
  var user2 = await Crypto.genSK();
  var packet = Packet(
      Crypto.derivePK(user1), Crypto.derivePK(user2),
      await Crypto.randBytes(100), Packet.zeroID);
  var zeroPacket = Packet(
      Crypto.derivePK(user1), Crypto.derivePK(user1), [], Packet.zeroID);
  List<Packet> packets = [zeroPacket];
  var timings = [];
  for (var i = 0; i < n; i++) {
    var startSend = DateTime.now();
    await Longlog.push(host, packet.encode(), user2, Duration(seconds: 2));
    var endSend = DateTime.now();
    var startRecv = DateTime.now();
    packets = await Longlog.pull(host, zeroPacket.encode().sublist(0,1+Packet.pkSize+8), 10, Duration(seconds: 2));
    var endRecv = DateTime.now();
    assert(packets.length == math.min(i + 1, 10));
    for (var pack in packets) {
      assert(Utils.equal(pack.data, packet.data));
      assert(Utils.equal(pack.from, packet.from));
      assert(Utils.equal(pack.to, packet.to));
    }
    timings.add({
      "send": endSend.difference(startSend).inMicroseconds,
      "recv": endRecv.difference(startRecv).inMicroseconds,
    });
    clearConsole();
    print("${((i + 1) / n * 100).toStringAsFixed(2)}%");
  }
  clearConsole();
  print(
      "send ${(timings.map((e) => e["send"]).reduce((value, element) => value + element) / timings.length / Duration.microsecondsPerSecond as double).toStringAsPrecision(4)}ms ${timings.map((e) => e["send"] / Duration.microsecondsPerSecond).toList()}");
  print(
      "recv ${(timings.map((e) => e["recv"]).reduce((value, element) => value + element) / timings.length / Duration.microsecondsPerSecond as double).toStringAsPrecision(4)}ms ${timings.map((e) => e["recv"] / Duration.microsecondsPerSecond).toList()}");
}
