

class LookupCacheEntry<T>{
  final DateTime deadline;
  final T value;
  LookupCacheEntry(this.deadline,this.value);
  bool isExpired(){
    return deadline.isBefore(DateTime.now());
  }
}

class LookupCache<T>{
  final Duration ttl;
  LookupCache(this.ttl);
  final Map<String,LookupCacheEntry<T>> _map = {};
  T? get(String key){
    var entry = _map[key];
    if (entry == null){
      return null;
    }
    if (entry.isExpired()){
      return null;
    }
    return entry.value;
  }
  clean(){
    _map.removeWhere((key, value) => value.isExpired());
  }
  void set(String key,T value,DateTime deadline){
    if(deadline.isBefore(DateTime.now())){
      return;
    }
    _map[key] = LookupCacheEntry<T>(deadline, value);
  }
}