import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'packet.dart';
import 'crypto.dart';
import 'net_addr.dart';
import 'package:messagepack/messagepack.dart' as msgpack;
import 'pool.dart';

var clientPool = Pool<HttpClient>(5, () {
  var cli = HttpClient();
  cli.autoUncompress = false;
  cli.maxConnectionsPerHost = 1;
  cli.userAgent = "dart";
  cli.idleTimeout = Duration(seconds: 5);
  return cli;
});

class Longlog {
  static Future<List<Packet>> pull(
      NetAddr host, List<int> last, int limit, Duration timeout) async {
    List<int> body;
    var cli = clientPool.pull();
    try {
      cli.connectionTimeout = timeout;
      var req = await cli
          .put(host.host, host.port, "/?limit=${limit.toString()}")
          .timeout(timeout);
      req.persistentConnection = true;
      req.bufferOutput = false;
      req.contentLength = last.length;
      req.add(last);
      var resp = await req.close().timeout(timeout);
      switch (resp.statusCode) {
        case 200:
          break;
        case 204:
          return [];
        default:
          var msg = await resp
              .reduce((previous, element) => previous..addAll(element))
              .timeout(timeout);
          throw Exception(msg.isEmpty ? resp.statusCode : utf8.decode(msg));
      }
      body = await resp
          .reduce((previous, element) => previous..addAll(element))
          .timeout(timeout);
      clientPool.push(cli);
    } catch (_) {
      cli.close(force: true);
      rethrow;
    }
    var pack = msgpack.Unpacker.fromList(body);
    var len = pack.unpackListLength();
    List<Packet> result = [];
    for (var i = 0; i < len; i++) {
      result.add(Packet.decode(pack.unpackBinary()));
    }
    return result;
  }

  static Future<void> push(
      NetAddr addr, List<int> data, List<int> sk, Duration timeout) async {
    var sig = base64UrlEncode(await Crypto.sign(sk, data));
    var cli = clientPool.pull();
    try {
      cli.connectionTimeout = timeout;
      var req = await cli.post(addr.host, addr.port, "/").timeout(timeout);
      req.persistentConnection = true;
      req.headers.set("Content-Signature", sig);
      req.bufferOutput = false;
      req.contentLength = data.length;
      req.add(data);
      var resp = await req.close().timeout(timeout);
      if (resp.statusCode != 204) {
        var body = await resp
            .reduce((previous, element) => previous..addAll(element))
            .timeout(timeout);
        throw Exception(body.isEmpty ? resp.statusCode : utf8.decode(body));
      }
      clientPool.push(cli);
    } catch(_) {
      cli.close(force: true);
      rethrow;
    }
  }
}
